load File.join(File.dirname(__FILE__),  "..", "test_helper.rb")

# These lines are comments (starting with #)
#

describe "Flight Reservation" do
  include TestHelper
  
  before(:all) do
    @browser = Watir::Browser.new
  end

  after(:all) do
    @browser.close unless debugging?
  end

  before(:each) do
    @browser.goto(site_url)
    browser.text_field(:name, "username").set("agileway")
    browser.text_field(:name, "password").set("testwise")
    browser.button(:value,"Sign in").click
  end

  after(:each) do
    # don't sign off when $TESTWISE_RUNNING_AS == "test_case" means running individual test case
    # If running all test cases in the test script file
    failsafe { browser.link(:text, "Sign off").click } unless debugging?
  end

  after(:all) do
    # Unless $TESTWISE_DEBUGGING flag is set, the tool bar Pin icon (right next to Firefox)
    # It is very useful to leave last web page open for analysis
    browser.close unless debugging?
  end

  # This test script is recorded using TestWise Recorder Plugin for Firefox plus
  # assertions (or check points in QTP's term).
  it "(Recorded version)  Book Flight" do
    browser.select_list(:name, "fromPort").select("New York")
    browser.select_list(:id, "departMonth").select("September 2012")
    browser.select_list(:name, "toPort").select("Sydney")
    browser.select_list(:id, "returnMonth").select("October 2012")
    browser.select_list(:name, "returnDay").select("05")
    browser.button(:value, "Continue").click

    browser.text_field(:name, "passengerFirstName").set "Bob"
    browser.text_field(:name, "passengerLastName").set "Builder"
    browser.button(:value, "Next").click    
    browser.text.should include("New York to Sydney")
  end

  # Refactor the above test to a more maintainable way by moving
  # operations to xxx_page.rb under pages/ folder, so that
  #  * if modfications (who doesn't) made to web pages, just need to change once
  #  * define you own testing syntax
  #  * TestWise IDE take advantage of this design/debug to make devleoping test easier
  #
  # This test case script is same as the one in agile_travel\test\watir folder, which uses
  #  RWebSpec syntax.
  # This design make your tests independent.
  # 
  it "(Refactored version): Book flight using Page object design" do
    flight_finder_page =  FlightFinderPage.new(browser)
    flight_finder_page.select_trip_type_return
    flight_finder_page.select_departing_from("New York")
    flight_finder_page.select_arriving_in("Sydney")
    flight_finder_page.select_departing_day("02")
    flight_finder_page.select_departing_month("September 2012")
    flight_finder_page.select_returning_day("05")
    flight_finder_page.select_returning_month("October 2012")

    passenger_page = flight_finder_page.click_continue
    passenger_page.enter_first_name "Wendy"
    passenger_page.enter_last_name "The Boss"
    passenger_page.click_next

    browser.text.should include("return New York to Sydney")
  end


end
